#include "HumanSystemModel.h"

namespace armarx::navigation::components::dynamic_scene_provider::kalman_filter
{

    // ------------------------------ SO2xR2 -----------------------------------------

    template <typename floatT>
    typename SystemModelSO2xR2<floatT>::ObsT
    SystemModelSO2xR2<floatT>::observationFunction(const SystemModelSO2xR2::StateT& state)
    {
        ObsT observation;
        observation.segment(0, 2) = state.position;
        observation.segment(2, 1) = state.orientation.log().coeffs();
        return observation;
    }

    template <typename floatT>
    typename SystemModelSO2xR2<floatT>::StateT
    SystemModelSO2xR2<floatT>::propagationFunction(const SystemModelSO2xR2::StateT& state,
                                                   const SystemModelSO2xR2::ControlT& control,
                                                   const SystemModelSO2xR2::ControlNoiseT& noise,
                                                   FloatT dt)
    {
        StateT new_state;
        new_state.orientation = state.orientation.template rplus(control.angular_velocity * dt);
        new_state.position = state.position + control.euclidean_velocity * dt;
        return new_state;
    }

    template <typename floatT>
    typename SystemModelSO2xR2<floatT>::StateT
    SystemModelSO2xR2<floatT>::retraction(const SystemModelSO2xR2::StateT& state,
                                          const SystemModelSO2xR2::SigmaPointsT& sigmaPoints)
    {
        StateT new_state;
        manif::SO2Tangent<FloatT> tan;
        tan.coeffs() << sigmaPoints.segment(2, 1);
        new_state.orientation = state.orientation.lplus(tan);

        new_state.position = state.position + sigmaPoints.segment(0, 2);
        return new_state;
    }

    template <typename floatT>
    typename SystemModelSO2xR2<floatT>::SigmaPointsT
    SystemModelSO2xR2<floatT>::inverseRetraction(const SystemModelSO2xR2::StateT& state1,
                                                 const SystemModelSO2xR2::StateT& state2)
    {
        SigmaPointsT sigma;
        // TODO: check if right order of substraction
        sigma.segment(0, 2) = state1.position - state2.position;
        sigma.segment(2, 1) = state2.orientation.lminus(state1.orientation).coeffs();
        return sigma;
    }

    template struct SystemModelSO2xR2<float>;
    template struct SystemModelSO2xR2<double>;

} // namespace armarx::navigation::components::dynamic_scene_provider::kalman_filter
