#pragma once


#include <ArmarXCore/core/time.h>

#include <RobotAPI/libraries/ukfm/UnscentedKalmanFilter.h>

#include <VisionX/libraries/armem_human/types.h>

#include <armarx / navigation / components / dynamic_scene_provider / HumanSystemModel.h>
#include <armarx/navigation/core/basic_types.h>
#include <armarx/navigation/human/types.h>

namespace armarx::navigation::components::dynamic_scene_provider
{
    using T = double; //TODO double or float?
    using Vector = Eigen::Matrix<T, 2, 1>;
    using SystemModelT = kalman_filter::SystemModelSO2xR2<T>;

    class HumanTracker
    {
    public:
        HumanTracker() = default;

        struct Measurements
        {
            DateTime detectionTime;
            std::vector<armem::human::HumanPose> humanPoses;
        };

        struct DetectedHuman
        {
            core::Pose2D pose;
            std::optional<std::string> trackingId;
            DateTime detectionTime;
            bool associated;
        };

        struct TrackedHuman
        {
            human::Human human;
            UnscentedKalmanFilter<SystemModelT> ukf;
            std::optional<std::string> trackingId = std::nullopt;
            bool associated;
        };

        struct Parameters
        {
            // the duration after which tracked humans will be erased if no new measurement for this human is found
            Duration maxTrackingAge = Duration::MilliSeconds(500);
            // the maximum distance in millimeters of two human measurements to be associated with each other
            float maxAssociationDistance = 600;
            // alpha value from interval [0,1] to determine how much the new (and respectively the old) velocity should be weighted
            float velocityAlpha = 0.7;
        };

        void update(const Measurements& measurements);

        std::vector<human::Human> getTrackedHumans() const;

        void reset();

    private:
        void associateHumans(std::vector<DetectedHuman>& detectedHumans);
        void associate(TrackedHuman* tracked, DetectedHuman* detected);

    private:
        std::vector<TrackedHuman> trackedHumans;
        Parameters parameters;
    };
} // namespace armarx::navigation::components::dynamic_scene_provider
