/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Tobias Gröger ( tobias dot groeger at student dot kit dot edu )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */
#pragma once

#include <armarx/navigation/human/types.h>

namespace armarx::navigation::human
{

    class ProxemicZoneCreator
    {

    public:
        struct ProxemicParameters
        {
            float intimateWeight = 2.0;
            float personalWeight = 1.0;

            float intimateRadius = 400;
            float personalRadius = 1000;

            float movementInfluence = 1;
            // in the coordinate system of the human
            Eigen::Vector2f offset{100, 150};
        };


        ProxemicZoneCreator()
        {
            intimatePenalty = {.minDistance = 0.1, .epsilon = 0, .exponent = 1.2};
            personalPenalty = {.minDistance = 0.1, .epsilon = 0, .exponent = 1.2};
        }
        ProxemicZoneCreator(ProxemicParameters& params,
                            ExponentialPenaltyModel& intimatePenalty,
                            ExponentialPenaltyModel& personalPenalty) :
            params(params), intimatePenalty(intimatePenalty), personalPenalty(personalPenalty)
        {
        }


        ProxemicZones createProxemicZones(const Human& human);

    private:
        ProxemicParameters params;
        ExponentialPenaltyModel intimatePenalty;
        ExponentialPenaltyModel personalPenalty;
    };

} // namespace armarx::navigation::human
