/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Tobias Gröger ( tobias dot groeger at student dot kit dot edu )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */
#pragma once

#include <VirtualRobot/CollisionDetection/CollisionModel.h>

#include <RobotAPI/components/ArViz/Client/Layer.h>

#include <armarx/navigation/human/ProxemicZoneCreator.h>
#include <armarx/navigation/human/types.h>
#include <teb_local_planner/obstacles.h>

namespace armarx::navigation::local_planning
{


    class TebObstacleManager
    {
    public:
        TebObstacleManager(teb_local_planner::ObstContainer& container) : container(container)
        {
        }

        void clear();

        size_t size();

        void addBoxObstacle(const VirtualRobot::BoundingBox& bbox, viz::Layer* visLayer = nullptr);
        void addHumanObstacle(const human::Human& human, viz::Layer* visLayer = nullptr);

    private:
        teb_local_planner::ObstContainer& container;
        human::ProxemicZoneCreator proxemics;
        int visualizationIndex;
    };


} // namespace armarx::navigation::local_planning
